#!/usr/bin/env python3
"""Read IV files and create plots.

Analisis de la IV con macros de la webApp

SENSOR_IV_Analysis.py in
https://gitlab.cern.ch/atlas-itk/sw/db/production_database_scripts.git

webApp aqui:
https://itk-pdb-webapps-strips.web.cern.ch

"""
import sys
import os
import json
import tempfile
import copy
from pathlib import Path

from matplotlib.backends.backend_gtk3agg import FigureCanvasGTK3Agg as FigureCanvas
from matplotlib.backends.backend_gtk3 import NavigationToolbar2GTK3 as NavigationToolbar
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt

import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils, UploadTest, SensorUtils

# Check if Gtk can be open
gtk_runs, gtk_args = Gtk.init_check()


def remove_files(W, flist):
    """Remove files given in the input list.

    Args:
        flist (list): list of filenames.

    """
    for f in flist:
        os.unlink(f)


class IVwindow(dbGtkUtils.ITkDBWindow):
    """GUI for IV file handling."""

    def __init__(self, session, title="IV window", options=None, help_link=None):
        """Initialization."""
        super().__init__(session=session, title=title,
                         show_search=None, gtk_runs=gtk_runs,
                         help_link=help_link
                         )
        self.mdata = {}
        self.mod_type = {}
        self.mod_SN = {}
        self.last_folder = None
        self.difference = None
        self.canvas = None

        self.init_window()

    def init_window(self):
        """Prepare the Gtk window."""
        self.hb.props.title = "IV data"

        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="view-refresh-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to refresh canvas.")
        button.connect("clicked", self.on_refresh)
        self.hb.pack_end(button)

        # Button to upload
        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="document-send-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to upload test")
        button.connect("clicked", self.do_upload)
        self.hb.pack_end(button)

        # File entry and search button
        self.single_file = Gtk.FileChooserButton()
        self.single_file.connect("file-set", self.on_single_file_set)

        self.double_file = Gtk.FileChooserButton()
        self.double_file.connect("file-set", self.on_double_file_set)

        self.single_SN = Gtk.Label(label="(None)")
        self.double_SN = Gtk.Label(label="(None)")

        grid = Gtk.Grid(column_spacing=5, row_spacing=1)

        grid.attach(Gtk.Label(label="Files"), 1, 0, 1, 1)
        grid.attach(Gtk.Label(label="Serial No."), 2, 0, 1, 1)

        grid.attach(Gtk.Label(label="Single Data File"), 0, 1, 1, 1)
        grid.attach(self.single_file, 1, 1, 1, 1)
        grid.attach(self.single_SN, 2, 1, 1, 1)

        grid.attach(Gtk.Label(label="Double Data File"), 0, 2, 1, 1)
        grid.attach(self.double_file, 1, 2, 1, 1)
        grid.attach(self.double_SN, 2, 2, 1, 1)

        #btn = Gtk.Button(label="Compute difference")
        #btn.connect("clicked", self.on_difference)
        #grid.attach(btn, 1, 3, 1, 1)

        btn = Gtk.Button(label="Upload to DB")
        btn.connect("clicked", self.do_upload)
        grid.attach(btn, 2, 3, 1, 1)

        self.mainBox.pack_start(grid, False, True, 0)

        self.fig = mpl.figure.Figure()
        self.fig.tight_layout()
        sw = Gtk.ScrolledWindow()  # Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

        # A scrolled window border goes outside the scrollbars and viewport
        sw.set_border_width(10)
        sw.set_size_request(310, 310)

        self.canvas = FigureCanvas(self.fig)  # a Gtk.DrawingArea
        self.canvas.set_size_request(400, 300)
        sw.add(self.canvas)
        self.mainBox.pack_start(sw, True, True, 0)

        # Create toolbar
        try:
            toolbar = NavigationToolbar(self.canvas)
        except TypeError:
            toolbar = NavigationToolbar(self.canvas, self)

        self.mainBox.pack_start(toolbar, False, False, 0)

        # The text view
        self.mainBox.pack_start(self.message_panel.frame, True, True, 5)

        self.show_all()

    def get_difference_data(self):
        """Returns the double data witht the difference."""
        # Prepare the data
        ddata = copy.deepcopy(self.mdata["double"])

        ndata = len(self.difference)
        ddata["curve"]["V"] = np.abs(ddata["curve"]["V"][:ndata])
        ddata["curve"]["I"] = np.abs(self.difference)
        ddata["curve"]["S"] = ddata["curve"]["S"][:ndata]
        return ddata

    def upload_test(self, mdata, mod_type):
        """Upload available tests."""

        # Get JSon skeleton filled
        test = SensorUtils.sensor_data_to_json(self.session, mdata, mod_type, self)

        # write attachment.
        # First geet the fine name.
        fnam = SensorUtils.build_file_name(mdata)


        data_out = tempfile.NamedTemporaryFile("w", prefix=fnam, suffix=".dat", delete=False)
        SensorUtils.save_sensor_data(data_out, mdata, name=fnam)

        js_out = tempfile.NamedTemporaryFile("w", prefix="payload-", suffix=".json", delete=False)
        js_out.write(json.dumps(test, indent=3, cls=dbGtkUtils.MyEncoder))
        js_out.close()

        attachment = ITkDButils.Attachment(path=data_out.name, title="resultsFile", desc=fnam)
        uploadW = UploadTest.UploadTest(self.session, js_out.name, attachment)
        uploadW.connect("destroy", remove_files, [data_out.name, js_out.name])


    def do_upload(self, *args):
        """The upload button has been clicked.

        We present a dialog where we ask if the new data file should be stored
        locally and if both (single and difference) tests should be uploaded.
        """
        if "single" not in self.mdata:
            return

        if "double" not in self.mdata:
            # upload only the single test.
            if dbGtkUtils.ask_for_confirmation(
                "Uploading Single data",
                "No data for double module/sensor.\nUpload single test ?."):

                self.upload_test(self.mdata["single"], self.mod_type["single"])

            return

        # We create the dialog.
        dlg = Gtk.Dialog(title="Add Attachment", parent=self, flags=0)
        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_OK, Gtk.ResponseType.OK)
        area = dlg.get_content_area()
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        area.add(grid)

        label = Gtk.Label(label="Save locally new data file ?")
        save_locally = Gtk.Switch()
        grid.attach(label, 0, 0, 1, 1)
        grid.attach(save_locally, 1, 0, 1, 1)

        label = Gtk.Label(label="Upload both tests ?")
        do_both = Gtk.Switch()
        grid.attach(label, 0, 1, 1, 1)
        grid.attach(do_both, 1, 1, 1, 1)

        dlg.show_all()
        rc = dlg.run()
        dlg.hide()
        if rc != Gtk.ResponseType.OK:
            dlg.destroy()
            return

        mdata = self.get_difference_data()
        if save_locally.get_active():
            # Save locally.
            fnam = SensorUtils.build_file_name(mdata)
            fc = Gtk.FileChooserDialog(title="Save data file", action=Gtk.FileChooserAction.SAVE)
            fc.add_buttons(
                Gtk.STOCK_CANCEL,
                Gtk.ResponseType.CANCEL,
                Gtk.STOCK_OPEN,
                Gtk.ResponseType.OK,
            )

            if self.last_folder:
                fc.set_current_folder(self.last_folder)

            fc.set_current_name("{}.dat".format(fnam))
            rc = fc.run()
            if rc == Gtk.ResponseType.OK:
                SensorUtils.save_sensor_data(fc.get_filename(), mdata)

            fc.hide()
            fc.destroy()

        # Upload double
        self.upload_test(mdata, self.mod_type["double"])

        if do_both.get_active():
            self.upload_test(self.mdata["single"], self.mod_type["single"])

        dlg.destroy()

    def on_refresh(self, *args):
        """Refresh canvas."""
        if self.fig and self.canvas:
            self.fig.tight_layout()
            self.canvas.draw()

    def find_module(self, SN):
        """Find module (SN) on database

        Args:
            SN (str): Module Serial number.

        """
        md = ITkDButils.get_DB_component(self.session, SN)
        if md is None:
            dbGtkUtils.complain(
                "Could not find {}".format(SN), str(ITkDButils.get_db_response())
            )

        return md

    def update_folder(self, fnam):
        """Sets last folder."""

        self.last_folder = Path(fnam).parent.as_posix()
        self.single_file.set_current_folder(self.last_folder)
        self.double_file.set_current_folder(self.last_folder)

    def show_single_curve(self):
        """Shows the single curve."""
        try:
            mdata = self.mdata["single"]
        except KeyError:
            return

        is_module = is_module = "Module_SN" in mdata

        self.show_curve(
            131,
            mdata["curve"]["V"],
            mdata["curve"]["I"],
            self.mod_type["single"][0:4] if is_module else "Single",
            mdata["curve"]["labels"][0],
            mdata["curve"]["labels"][1],
        )

    def show_double_curve(self):
        """Shows the double curve."""
        try:
            mdata = self.mdata["double"]
        except KeyError:
            return

        self.show_curve(
            133,
            mdata["curve"]["V"],
            mdata["curve"]["I"],
            "Double",
            mdata["curve"]["labels"][0],
            None,
        )

    def on_single_file_set(self, *args):
        """Single sensor file chosen."""
        obj_type = ["sensor", "module"]
        fnam = self.single_file.get_filename()
        if fnam is None or not Path(fnam).exists():
            dbGtkUtils.complain("Could not find data file", fnam, parent=self)

        mdata = SensorUtils.read_sensor_file(fnam)
        self.update_folder(fnam)

        is_module = 1
        try:
            SN = mdata["Module_SN"]

        except KeyError:
            SN = mdata["Component"]
            is_module = 0

        self.write_message("Reading data for {} {}\n".format(obj_type[is_module], SN))
        md = self.find_module(SN)
        if md is None:
            self.write_message("...object does not exist.\n")
            self.single_file.unselect_all()
            return

        # All good
        self.mod_SN["single"] = SN
        self.mdata["single"] = mdata
        self.mod_type["single"] = md["type"]["code"]
        print(self.mod_type["single"])

        self.single_SN.set_text("{} - {}".format(SN, md["type"]["name"]))
        self.fig.clf()
        self.show_single_curve()

        # Compute difference if single already available
        if "double" in self.mdata:
            self.show_double_curve()
            self.on_difference()

    def check_double_SN(self, SN, is_module):
        """Check that the double SN is a good one."""
        obj_type = ["sensor", "module"]
        if "single" in self.mod_SN:
            if self.mod_SN["single"] == SN:
                dbGtkUtils.complain(
                    "Wrong SN {}".format(SN),
                    "{} already used.".format(obj_type[is_module])
                )
                self.double_file.unselect_all()
                return None

        # Check that it exists in the DB
        if len(SN) != 14 or SN[0:4] != "20US":
            self.write_message("Invalid SN: {}\n".format(SN))
            SN = dbGtkUtils.get_a_value(
                "Invalid SN", "Give Ring or corresponding Half Module SN"
            )
            return None

        md = self.find_module(SN)
        if md is None:
            self.write_message("...object does not exist.\n")
            self.double_file.unselect_all()
            return None

        return md

    def get_true_SN(self, md):
        """Get the actual SN of the 'double' object."""
        found_child = False
        has_ring = md["type"]["name"].find("Ring")
        if has_ring >= 0:
            self.write_message("...This is a Ring module. Searching children in DB\n")
            for child in md["children"]:
                if child["component"]:
                    ctype = child["type"]["code"]
                    if ctype.find("MODULE") < 0:
                        continue

                    cSN = child["component"]["serialNumber"]
                    if cSN == self.mod_SN["single"]:
                        continue

                    halfM_SN = cSN
                    found_child = True
                    self.write_message("...found {}\n".format(halfM_SN))
                    break

            if not found_child:
                self.write_message("Requesting a Half Module SN\n")
                halfM_SN = dbGtkUtils.get_a_value(
                    "Give Half Module SN", "Serial Number"
                )

            md = ITkDButils.get_DB_component(self.session, halfM_SN)
            if md is None:
                dbGtkUtils.complain(
                    "Could not find {}".format(halfM_SN),
                    str(ITkDButils.get_db_response()),
                )
                self.double_file.unselect_all()
                return None

            self.write_message("... {}\n".format(halfM_SN))
            return halfM_SN

        return md["serialNumber"]

    def on_double_file_set(self, *args):
        "File chosen for the 'double module'"
        obj_type = ["sensor", "module"]
        fnam = self.double_file.get_filename()
        if fnam is None or not Path(fnam).exists():
            dbGtkUtils.complain("Could not find data file", fnam, parent=self)

        mdata = SensorUtils.read_sensor_file(fnam)
        self.update_folder(fnam)
        is_module = 1
        # Check SN in data file
        try:
            SN = mdata["Module_SN"]
        except KeyError:
            is_module = 0
            SN = mdata["Component"]

        halfM_SN = SN
        md = self.check_double_SN(SN, is_module)
        if md is None:
            return

        self.write_message("Reading data for {} {}\n".format(obj_type[is_module], SN))
        halfM_SN = self.get_true_SN(md)
        if halfM_SN is None:
            return

        if "single" in self.mod_type:
            if is_module and self.mod_type["single"] == md["type"]["code"]:
                dbGtkUtils.complain(
                    "Wrong module type.",
                    "Module type cannot be {}".format(self.mod_type["single"]),
                )

                self.double_file.unselect_all()
                return

        self.mod_SN["double"] = halfM_SN
        self.mod_type["double"] = md["type"]["code"]
        self.mdata["double"] = mdata

        self.double_SN.set_text("{} - {}".format(halfM_SN, md["type"]["name"]))
        self.show_double_curve()

        # Compute difference if single already available
        if "single" in self.mdata:
            self.on_difference()

    def on_difference(self, *args):
        """Compute difference."""
        if "single" not in self.mdata or "double" not in self.mdata:
            dbGtkUtils.complain(
                "Data needed", "Check if single oand doubel module data are available"
            )
            return

        is_module = "Module_SN" in self.mdata["double"]
        double_I = self.mdata["double"]["curve"]["I"]
        single_I = SensorUtils.scale_iv(
            self.mdata["single"]["curve"]["I"],
            self.mdata["single"]["Temperature"] + 273.0,
            self.mdata["double"]["Temperature"] + 273.0,
        )

        try:
            nmin = double_I.size
            self.difference = double_I - single_I
        except ValueError:
            nmin = np.min([double_I.size, single_I.size])
            self.write_message(
                "Size of current arrays is not the same: {} {}\n".format(
                    double_I.size, single_I.size
                )
            )
            self.difference = double_I[:nmin] - single_I[:nmin]

        self.show_curve(
            132,
            self.mdata["double"]["curve"]["V"][:nmin],
            self.difference,
            self.mod_type["double"][0:4] if is_module else "Diff",
            self.mdata["double"]["curve"]["labels"][0],
            None,
        )

    def show_curve(self, subplot, X, Y, title=None, xlabel="X", ylabel="Y"):
        """Shows data"""
        ax = self.fig.add_subplot(subplot)
        plt.cla()
        if xlabel:
            ax.set_xlabel(xlabel)

        if ylabel:
            ax.set_ylabel(ylabel)

        if title:
            ax.set_title(title)

        ax.plot(X, Y)
        ax.grid()
        self.on_refresh()


def main():
    """Main entryy."""

    # DB login
    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg

    # Start the Application
    win = IVwindow(client)
    win.show_all()
    win.set_accept_focus(True)
    win.present()
    win.connect("destroy", Gtk.main_quit)

    try:
        Gtk.main()
    except KeyboardInterrupt:
        print("Arrggggg !!!")

    dlg.die()
    print("Bye !!")
    sys.exit()

if __name__ == "__main__":
    main()



import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, GObject, Gio, GLib, GdkPixbuf




dlg = Gtk.Dialog(title="test animated", flags=0)

dlg.add_buttons(Gtk.STOCK_OK, Gtk.ResponseType.OK)

area = dlg.get_content_area()
area.set_size_request(-1, 200)


#overlay = Gtk.Overlay.new()
#area.add(overlay)

lbl = Gtk.Label(label="The Label")
area.pack_start(lbl, True, True, 0)

pxbuf = GdkPixbuf.PixbufAnimation.new_from_file("/Users/lacasta/Downloads/ezgif-3-9b8ffa642a.gif")

img = Gtk.Image.new_from_animation(pxbuf)
img.set_valign(Gtk.Align.CENTER)
img.set_halign(Gtk.Align.CENTER)
#overlay.add_overlay(img)

area.add(img)

dlg.show_all()
dlg.run()





try:
    import itkdb_gtk
    
except ImportError:
    import sys
    from pathlib import Path
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, Gdk


class ShowDefects(Gtk.Dialog):
    """Edit defects"""
    
    def __init__(self, title, defects=None, parent=None):
        """Initialization."""
        super().__init__(title=title, transient_for=parent)
        if defects:
            self.defects = [C for C in defects]
        else:
            self.defects = []
            
        self.init_window()

    def init_window(self):
        """Preapare the window."""
        self.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                         Gtk.STOCK_OK, Gtk.ResponseType.OK)

        self.mainBox = self.get_content_area()
        # The "Add attachment" button.
        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.mainBox.pack_start(box, False, False, 0)

        dbGtkUtils.add_button_to_container(box, "Add Defect",
                                           "Click to add a new defect.",
                                           self.add_defect)

        dbGtkUtils.add_button_to_container(box, "Remove Defect",
                                           "Click to remove selected defect.",
                                           self.remove_defect)

        # the list of attachments
        tree_view = self.create_tree_view()
        self.mainBox.pack_start(tree_view, True, True, 0)
        for C in self.defects:
            self.append_defect_to_view(C)

        self.show_all()

    def create_tree_view(self, size=150):
        """Create the tree view."""
        model = Gtk.ListStore(str, str)
        self.tree = Gtk.TreeView(model=model)
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.tree)
        scrolled.set_size_request(-1, size)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Defect Type", renderer, text=0)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Description", renderer, text=1)
        self.tree.append_column(column)

        return scrolled

    def add_defect(self, *args):
        """A new defect."""
        values = dbGtkUtils.get_a_list_of_values("Insert new defect", ("Type", "Description/v"))
        if len(values):
            defect = {"name": values[0], "description": values[1]}
            self.defects.append(defect)
            self.append_defect_to_view(defect)

    def remove_defect(self, *args):
        """Remove a defect."""
        select = self.tree.get_selection()
        model, iter = select.get_selected()
        if iter:
            values = model[iter]
            for C in self.defects:
                if C["name"] == values[0] and C["description"] == values[1]:
                    rc = dbGtkUtils.ask_for_confirmation("Remove this Defect ?",
                                                         "{}\n{}\n".format(values[0], values[1]))
                    if rc:
                        self.defects.remove(C)
                        model.remove(iter)

                    break

    def append_defect_to_view(self, C):
        """Append a new defect to the triee view."""
        model = self.tree.get_model()
        model.append([C["name"], C["description"]])



#!/usr/bine/env python3
"""test big attachments."""

import json
import sys
from pathlib import Path
import itkdb

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import ITkDBlogin, ITkDButils


# DB login
dlg = ITkDBlogin.ITkDBlogin()
client = dlg.get_client(use_eos=True)
if client is None:
    print("Could not connect to DB with provided credentials.")
    dlg.die()
    sys.exit()

client.user_gui = dlg

ifile = Path("/Users/lacasta/Nextcloud/ITk/5-Petal_cores/PPC.007/Rx_PPC.007.png")
core = ITkDButils.get_DB_component(client, "PPC.007")
data = { "filterMap": { "serialNumber": core["serialNumber"], "testType":["XRAYIMAGING"]},
}

test_list = client.get("listTestRunsByComponent", json=data)
ntest = 0
for tst in test_list:
    if tst['state'] == 'ready':
        ntest += 1
        
if ntest == 0:
    defaults = {
        "institution": "IFIC",
        "runNumber": "1",
    }

    tmp = ITkDButils.get_test_skeleton(client, "CORE_PETAL", "XRAYIMAGING", defaults)
    tmp["component"] = core["serialNumber"]
    tmp["properties"]["OPERATOR"] = "Nicolas Cespedosa"
    tmp["properties"]["MACHINEID"] ="X-ray"
    print(tmp)




#client._use_eos = True 
A = ITkDButils.Attachment(path=ifile.as_posix(),  
                          title="{} X-ray".format(core["alternativeIdentifier"]),
                          desc="{} X-ray image".format(core["alternativeIdentifier"])
                            )
rc = ITkDButils.upload_test(client, data=tmp, attachments=[A])

if rc:
    ipos = rc.find("The following details may help:")
    print(rc[ipos:])
    
else:
    print("upload successful")
    
    
dlg.die()



#!/usr/bin/env python3
"""Module Visual inspection GUI."""
import sys
import re
from pathlib import Path

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils
from itkdb_gtk.ShowComments import ShowComments
from itkdb_gtk.ShowDefects import ShowDefects
from itkdb_gtk.ShowAttachments import ShowAttachments
from itkdb_gtk.UploadTest import UploadTest

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio


module_type = re.compile("20USE(M[0-5]{1}|[345]{1}[LR]{1})[0-9]{7}")
sensor_type = re.compile("20USES[0-5]{1}[0-9]{7}")


class ModuleVisualInspection(dbGtkUtils.ITkDBWindow):
    """Module/Sensor Visual Inspection."""

    def __init__(self, session, title="Visual Inspection", help_link=None):
        super().__init__(title=title,
                         session=session,
                         show_search="Find object with given SN.",
                         help_link=help_link)

        self.institute = self.pdb_user["institutions"][0]["code"]
        self.global_image = None
        self.global_link = None
        self.data = None
        self.currentStage = None
        self.attachments = []
        self.comments = []
        self.defects = []

        # action button in header
        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="document-send-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to upload the test.")
        button.connect("clicked", self.upload_test)
        self.hb.pack_end(button)

        grid = Gtk.Grid(column_spacing=10, row_spacing=5)
        self.mainBox.pack_start(grid, False, False, 5)

        irow = 0
        receiver = self.create_institute_combo(only_user=True)
        receiver.connect("changed", self.on_institute)
        receiver.set_tooltip_text("Select the Institute making the test.")
        dbGtkUtils.set_combo_iter(receiver, self.institute)

        lbl = Gtk.Label(label="Institute")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)
        grid.attach(receiver, 1, irow, 1, 1)

        self.obj_type = None
        self.obj_type_label = Gtk.Label()
        self.obj_type_label.set_xalign(0)
        grid.attach(self.obj_type_label, 2, irow, 1, 1)

        irow += 1
        lbl = Gtk.Label(label="Serial Number")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)

        self.SN = dbGtkUtils.TextEntry(small=True)
        self.SN.connect("text_changed", self.SN_ready)
        self.SN.widget.set_tooltip_text("Enter SN of module.")
        grid.attach(self.SN.widget, 1, irow, 1, 1)


        self.passed = Gtk.Switch()
        self.passed.props.halign = Gtk.Align.START
        self.passed.set_active(True)
        lbl = Gtk.Label(label="Passed")
        lbl.set_xalign(0)
        grid.attach(lbl, 2, irow, 1, 1)
        grid.attach(self.passed, 3, irow, 1, 1)


        irow += 1
        lbl = Gtk.Label(label="Date")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)

        self.date = dbGtkUtils.TextEntry(small=True)
        grid.attach(self.date.widget, 1, irow, 1, 1)
        self.date.entry.set_text(ITkDButils.get_db_date())
        self.date.connect("text_changed", self.new_date)

        self.problems = Gtk.Switch()
        self.problems.props.halign = Gtk.Align.START
        self.problems.set_active(False)
        lbl = Gtk.Label(label="Problems")
        lbl.set_xalign(0)
        grid.attach(lbl, 2, irow, 1, 1)
        grid.attach(self.problems, 3, irow, 1, 1)

        irow +=1
        self.reception = Gtk.Switch()
        self.reception.props.halign = Gtk.Align.START
        self.reception.set_active(False)
        lbl = Gtk.Label(label="Reception VI")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)
        grid.attach(self.reception, 1, irow, 1, 1)

        irow +=1
        lbl = Gtk.Label()
        grid.attach(lbl, 0, irow, 1, 1)

        # The "Add attachment" button.
        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.mainBox.pack_start(box, False, False, 0)
        self.btn_attch = dbGtkUtils.add_button_to_container(box, "Attachments",
                                           "Click to edit attachments.",
                                           self.edit_attachments)

        self.btn_comments = dbGtkUtils.add_button_to_container(box, "Comments",
                                           "Click to edit comments.",
                                           self.edit_comments)

        self.btn_defects = dbGtkUtils.add_button_to_container(box, "Defects",
                                           "Click to edit defects.",
                                           self.edit_defects)

        self.mainBox.pack_start(self.message_panel.frame, True, True, 5)
        self.write_message("Module Visual Inspection\n")
        self.show_all()

        dbGtkUtils.setup_scanner(self.get_qrcode)

    def on_institute(self, combo):
        """A new recipient has been chosen."""
        name = self.get_institute_from_combo(combo)
        if name:
            self.institute = name

    def new_date(self, entry, value):
        """new date given at input."""
        d = dbGtkUtils.parse_date_as_string(value)
        if d is not None:
            self.date.set_text(d)

    def SN_ready(self, *args):
        """SN is ready in the TextEnttry."""
        SN = self.SN.get_text()
        # GEt children.
        module = ITkDButils.get_DB_component(self.session, SN)
        if module is None:
            self.write_message(ITkDButils.get_db_response())
            return

        SN = module["serialNumber"]
        if module_type.match(SN):
            self.obj_type_label.set_markup("<b><big>Module</big></b>")
            self.obj_type = "MODULE"
        elif sensor_type.match(SN):
            self.obj_type_label.set_markup("<b><big>Sensor</big></b>")
            self.obj_type = "SENSOR"
        else:
            self.obj_type_label.set_text("Invalid SN")
            self.obj_type = None
            dbGtkUtils.complain("Invalid SN", "Not a module nor a sensor.")

        self.reception.set_sensitive(True)
        self.reception.set_active(False)
        self.currentStage = module["currentStage"]["code"]
        if self.obj_type == "MODULE" and self.currentStage in ["AT_LOADING_SITE", "STITCH_BONDING"]:
            self.reception.set_active(True)
            self.reception.set_sensitive(False)

        args[0].set_text(SN)


    def edit_attachments(self, *args):
        """Edit test attachmetns."""
        SA = ShowAttachments("Test Attachments", self.session, self.attachments, parent=self)
        response = SA.run()
        if response == Gtk.ResponseType.OK:
            self.attachments = SA.attachments

        SA.hide()
        SA.destroy()

        if len(self.attachments) > 0:
            self.btn_attch.set_label("Attachments ({})".format(len(self.attachments)))

    def edit_comments(self, *args):
        """Edit test comments."""
        SC = ShowComments("Test Comments", self.comments, self)
        rc = SC.run()
        if rc == Gtk.ResponseType.OK:
            self.comments = SC.comments

        SC.hide()
        SC.destroy()

        if len(self.comments) > 0:
            self.btn_comments.set_label("Comments ({})".format(len(self.comments)))

    def edit_defects(self, *args):
        """Edit test defects."""
        SD = ShowDefects("Test Defects", self.defects, self)
        rc = SD.run()
        if rc == Gtk.ResponseType.OK:
            self.defects = SD.defects

        SD.hide()
        SD.destroy()

        if len(self.defects) > 0:
            self.btn_defects.set_label("Defects ({})".format(len(self.defects)))

    def upload_test(self, *args):
        """Upload the test."""
        SN = self.SN.get_text()
        if len(SN) == 0 or self.obj_type is None:
            dbGtkUtils.complain("Invalid Serial Number", SN)
            return

        defaults = {
            "component": SN,
            "institution": self.institute,
            "passed": self.passed.get_active(),
            "problems": self.problems.get_active(),
            "runNumber": "1",
            "date": self.date.get_text()
        }

        test_type = "VISUAL_INSPECTION"
        if self.obj_type == "SENSOR":
            test_type = "VIS_INSP_RES_MOD_V2"

        else:
            if self.reception.get_active():
                test_type = "VISUAL_INSPECTION_RECEPTION"


        self.data = ITkDButils.get_test_skeleton(self.session,
                                                 self.obj_type,
                                                 test_type,
                                                 defaults)

        self.data["comments"] = self.comments
        self.data["defects"] = self.defects
        uploadW = UploadTest(self.session, self.data, self.attachments)



    def get_qrcode(self, fd, state, reader):
        """Read SN from scanner."""
        txt = dbGtkUtils.scanner_get_line(reader)
        self.write_message("SN: {}\n".format(txt))
        self.SN_ready(txt, self.SN.widget)


def main():
    """Main entry."""
    HELP_LINK="https://itkdb-gtk.docs.cern.ch/moduleVisualInspection.html"

    # DB login
    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg

    gTest = ModuleVisualInspection(client, help_link=HELP_LINK)

    gTest.present()
    gTest.connect("destroy", Gtk.main_quit)
    try:
        Gtk.main()

    except KeyboardInterrupt:
        print("Arrrgggg!!!")

    dlg.die()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""GEt shipments to a particular site (default is IFIC)."""
import sys
from pathlib import Path

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils
import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, GLib

# Check if Gtk can be open
gtk_runs, gtk_args = Gtk.init_check()


def find_vtrx(client, SN):
    """Searches VTRx."""
    payload = {
        "filterMap": {
            "project": "CE",
            "componentType": ["VTRX"],
            "propertyFilter": [{"code": "PACKAGE_SN", "operator": "=", "value": SN}],
        }
    }
    out = client.get("listComponentsByProperty", json=payload)
    vtrx = None
    nitem = 0
    for item in out:
        vtrx = item["serialNumber"]
        nitem += 1

    if nitem > 1:
        raise ValueError("Too many VTRx with same device SN.")

    return vtrx

class ReceiveShipments(dbGtkUtils.ITkDBWindow):
    """Find shipments related to given recipient."""

    def __init__(self, session, recipient=None, help_link=None):
        """Initialization.

        Args:
            session: ITkDB session.
            recipient: default recipient

        """
        self.state = "inTransit"
        self.institute = None
        self.model = None
        self.store = None
        self.tree = None
        self.shipments = {}

        global gtk_runs
        if gtk_runs:
            super().__init__(session=session, title="Receive Shipments",
                             show_search="Click to search shipments",
                             gtk_runs=gtk_runs, help_link=help_link)

            self.recipient = self.pdb_user["institutions"][0]["code"]
            self.init_window()
        else:
            pdb_user = ITkDButils.get_db_user(session)
            self.recipient = pdb_user["institutions"][0]["code"]

    def init_window(self):
        """Initialize window."""
        #
        self.set_border_width(10)

        # intercept keyboard
        dbGtkUtils.setup_scanner(self.get_qrcode)

        # Prepare HeaderBar
        self.hb.props.title = "{} shipments".format(self.recipient)

        # action button in header
        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="insert-object-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to receive shipment items.")
        button.connect("clicked", self.receive_items)
        self.hb.pack_end(button)

        # Data panel
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        self.mainBox.pack_start(grid, False, True, 0)

        # The shipment receiver
        receiver = self.create_institute_combo(only_user=True)
        receiver.connect("changed", self.on_receiver)
        receiver.set_tooltip_text("Select the Institute receiving the items.")
        dbGtkUtils.set_combo_iter(receiver, self.recipient)
        grid.attach(Gtk.Label(label="To:"), 0, 0, 1, 1)
        grid.attach(receiver, 1, 0, 1, 1)

        # The senders combo
        senders = self.create_institute_combo()
        senders.connect("changed", self.on_institute)
        senders.set_tooltip_text("Select the Institute sending the items.")
        grid.attach(Gtk.Label(label="From: "), 0, 1, 1, 1)
        grid.attach(senders, 1, 1, 1, 1)

        # The shipment status
        status = Gtk.ListStore(str)
        for st in ["prepared", "inTransit", "delivered", "deliveredIncomplete", "deliveredWithDamage", "undelivered"]:
            status.append([st])
        cmb_status = Gtk.ComboBox.new_with_model_and_entry(status)
        cmb_status.set_entry_text_column(0)
        cmb_status.set_active(1)
        cmb_status.connect("changed", self.on_status_changed)
        grid.attach(Gtk.Label(label="State"), 0, 2, 1, 1)
        grid.attach(cmb_status, 1, 2, 1, 1)

        # The list of shipments from sender to receiver
        shipment_list = Gtk.ListStore(str, str)
        cmb_shipment = Gtk.ComboBox.new_with_model_and_entry(shipment_list)
        cmb_shipment.set_entry_text_column(0)
        cmb_shipment.set_active(1)
        cmb_shipment.connect("changed", self.on_new_shipment)
        self.cmb_shipment = cmb_shipment
        grid.attach(Gtk.Label(label="Shipment"), 0, 3, 1, 1)
        grid.attach(cmb_shipment, 1, 3, 1, 1)

        btn = Gtk.Button(label="Receive Items")
        btn.set_tooltip_text("Click to receive shipment items.")
        btn.connect("clicked", self.receive_items)
        grid.attach(btn, 2, 3, 1, 1)

        self.check_all = Gtk.ToggleButton(label="Check A_ll", use_underline=True)
        self.check_all.set_active(1)
        self.check_all.set_tooltip_text("If toggled, items will need to be `received`\none by one by unchecking the  check box.")
        grid.attach(self.check_all, 3, 3, 1, 1)


        # Add a Separator
        self.mainBox.pack_start(Gtk.Separator(orientation=Gtk.Orientation.VERTICAL), False, True, 0)

        # Paned object
        paned = Gtk.Paned(orientation=Gtk.Orientation.VERTICAL)
        paned.set_size_request(-1, 200)
        self.mainBox.pack_start(paned, True, True, 0)

        # The main TreeView
        tree_view = self.create_tree_view()
        paned.add1(tree_view)

        # The text view
        paned.add2(self.message_panel.frame)

        self.show_all()

    def create_tree_view(self, size=150):
        """Create a tree view with the shipment items."""
        store = Gtk.ListStore(str, str, str, bool)
        self.tree = Gtk.TreeView(model=store)
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.tree)
        scrolled.set_size_request(-1, size)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Serial No.", renderer, text=0)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Object", renderer, text=1)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererToggle()
        column = Gtk.TreeViewColumn("UnCheck to receive", renderer, active=3)
        renderer.connect("toggled", self.on_cell_toggled)
        self.tree.append_column(column)

        return scrolled

    def get_qrcode(self, fd, state, reader):
        """Gets data from QR scanner."""
        txt = dbGtkUtils.scanner_get_line(reader)
        self.write_message("{}\n".format(txt))

        if txt.find("J-SD") == 0:
            try:
                txt = find_vtrx(self.session, txt)
            except ValueError as e:
                self.write_message("Error: {}\n".format(e))
                return

        # search code in the list
        if self.store:
            lv_iter = self.store.get_iter_first()
            while lv_iter:
                if self.store.get_value(lv_iter, 0) == txt:
                    self.write_message("...found\n")
                    self.store[lv_iter][3] = False

                lv_iter = self.store.iter_next(lv_iter)

        return True

    def on_cell_toggled(self, widget, path):
        """A cell has been toggled."""
        model = self.tree.get_model()
        model[path][3] = not model[path][3]

    def on_new_shipment(self, combo):
        """New shipment selected from combo."""
        if len(combo.get_model()) == 0:
            return

        tree_iter = combo.get_active_iter()
        if tree_iter is None:
            self.write_message("Cannot select current shipment.\n")
            return

        model = combo.get_model()
        shpmnt = model[tree_iter][1]

        # Store the model associated to this shipment.
        self.store = self.shipments[shpmnt]
        sfilter = self.store.filter_new()
        sfilter.set_visible_column(3)
        self.tree.set_model(sfilter)

    def on_status_changed(self, combo):
        """Status changed."""
        tree_iter = combo.get_active_iter()
        if tree_iter is not None:
            model = combo.get_model()
            name = model[tree_iter][0]
        else:
            entry = combo.get_child()
            name = entry.get_text()

        self.state = name

    def on_receiver(self, combo):
        """Sets the recipient."""
        name = self.get_institute_from_combo(combo)
        if name:
            self.recipient = name
            self.set_window_title("{} shipments".format(self.recipient))

    def on_institute(self, combo):
        """New institute chosen."""
        name = self.get_institute_from_combo(combo)
        if name:
            self.institute = name

    def get_tree_view_model(self):
        """Return an empty model for the tree view."""
        return Gtk.ListStore(str, str, str, bool, str, str)

    def query_db(self, *args):
        """Query for shipments in DB."""
        if self.state == "":
            return

        payload = {
            "filterMap": {
                "code": self.recipient,
                "status": self.state
            }
        }
        shpmts = self.session.get("listShipmentsByInstitution", json=payload)

        # We store SN, component type, shipment ID, visibility
        # We populate the combo of shipments and prepare the models for
        # the tree view.
        cmb_store = Gtk.ListStore(str, str)
        for s in shpmts:
            valid_sender = True
            if self.institute is not None:
                valid_sender = s['sender']['code'] == self.institute

            if s["recipient"]["code"] == self.recipient and valid_sender:
                store = self.get_tree_view_model()
                cmb_store.append([s['name'], s['id']])
                items = self.session.get("listShipmentItems", json={"shipment": s["id"]})
                for i in items:
                    store.append([i['component']['serialNumber'],
                                  i["component"]["componentType"]['name'],
                                  s['id'],
                                  True,
                                  s['name'],
                                  i['component']['code']])
                self.shipments[s['id']] = store

        # Load the model in the shipment combo
        if len(cmb_store) == 0:
            self.write_message("Could not find any shipment in DB.\n")

        self.cmb_shipment.set_model(cmb_store)
        if len(cmb_store)>0:
            self.cmb_shipment.set_entry_text_column(0)
            self.cmb_shipment.set_active(0)
        else:
            self.cmb_shipment.set_active(-1)
            self.cmb_shipment.get_child().set_text("")
            self.tree.set_model(Gtk.ListStore(str, str, str, bool))

    def mark_all_as_delivered(self):
        """Mark all items in current shipment as delivered."""
        if self.store is None:
            return

        lv_iter = self.store.get_iter_first()
        while lv_iter:
            val = self.store[lv_iter][3]
            self.store[lv_iter][3] = not val
            lv_iter = self.store.iter_next(lv_iter)

    def receive_items(self, *args):
        """Receive shipment items."""
        data = {}
        names = {}
        if not self.store:
            self.write_message("Empty list of items.\n")
            return

        if not self.check_all.get_active():
            self.mark_all_as_delivered()

        # self.store is the model of the tree view
        lv_iter = self.store.get_iter_first()
        while lv_iter:
            shpmnt = self.store.get_value(lv_iter, 2)
            if shpmnt not in data:
                data[shpmnt] = create_shipment_status(shpmnt)
                names[shpmnt] = self.store.get_value(lv_iter, 4)

            item = {
                "code": self.store[lv_iter][5],
                "delivered": not self.store[lv_iter][3]
            }
            data[shpmnt]["shipmentItems"].append(item)

            lv_iter = self.store.iter_next(lv_iter)


        for oid, S in data.items():
            # Check that all items are there
            nlost = 0
            for item in S["shipmentItems"]:
                if not item["delivered"]:
                    nlost += 1

            for Q in S["checklist"]["questionList"]:
                if Q["code"] == "Q14":
                    Q["value"] = (nlost == 0)

            if nlost:
                rc = dbGtkUtils.ask_for_confirmation("{} items are missing.".format(nlost),
                                                     "Proceed with shipment reception ?")
                if not rc:
                    self.write_message("Shipment reception cancelled by user.\n")
                    return

            # Open dialog to fill-in questions
            create_check_list(S["checklist"]["questionList"], names[oid])

            # send the update to the DB
            S['status'] = "delivered"
            resp = ITkDButils.set_shipment_status(self.session, S)
            if resp is None:
                rc = ITkDButils.get_db_response()
                ipos = rc.find("The following details may help:")
                msg = ""
                if ipos >= 0:
                    msg = rc[ipos:]

                dbGtkUtils.complain("Could not update the shipment status.", msg)
                self.write_message("Could not update the shipment status.\n{}\n".foramt(rc))

            else:
                self.write_message("Shipment {} received\n".format(names[oid]))
                # Now remove the current shipment
                lv_iter = self.cmb_shipment.get_active_iter()
                self.cmb_shipment.get_model().remove(lv_iter)
                self.cmb_shipment.set_active(0)


def create_check_list(check_list, name):
    """Create a checklist."""
    dlg = Gtk.Dialog(title="Question List: {}".format(name))
    dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_OK, Gtk.ResponseType.OK)
    grid = Gtk.Grid(column_spacing=5, row_spacing=1)
    box = dlg.get_content_area()
    box.add(grid)

    irow = 0
    S = {}
    T = {}
    for question in check_list:
        label = Gtk.Label(label=question["text"])
        label.set_xalign(0.0)
        if isinstance(question["value"], bool):
            sw = Gtk.Switch()
            sw.set_active(question["value"])
            S[question["code"]] = sw
            wdgt = Gtk.Box()
            wdgt.pack_start(sw, False, False, 0)
        else:
            wdgt = Gtk.Entry()
            wdgt.set_text(question["value"])
            T[question["code"]] = wdgt

        grid.attach(label, 0, irow, 1, 1)
        grid.attach(wdgt, 1, irow, 1, 1)
        irow += 1

    dlg.show_all()
    rc = dlg.run()
    if rc == Gtk.ResponseType.OK:
        for question in check_list:
            if isinstance(question["value"], bool):
                question["value"] = S[question["code"]].get_active()

            else:
                question["value"] = T[question["code"]].get_text()

    dlg.hide()
    dlg.destroy()
    return check_list


def create_shipment_status(shpmnt):
    """Create the json data for a shipment status update."""
    out = {
        "shipment": shpmnt,
        "status": "delivered",
        "shipmentItems": [],
        "checklist": {"type": "DEFAULT",
                      "questionList": [
                          {
                              "code": "Q01",
                              "text": "Shipment was properly packaged",
                              "value": False
                          },
                          {
                              "code": "Q02",
                              "text": "Outer box intact",
                              "value": False
                          },
                          {
                              "code": "Q03",
                              "text": "Inner box intact",
                              "value": False
                          },
                          {
                              "code": "Q04",
                              "text": "ESD state bag used",
                              "value": False
                          },
                          {
                              "code": "Q05",
                              "text": "Humidity barrier bag used",
                              "value": False
                          },
                          {
                              "code": "Q06",
                              "text": "Vacuum sealed bag used",
                              "value": False
                          },
                          {
                              "code": "Q07",
                              "text": "50g shock watch present",
                              "value": False
                          },
                          {
                              "code": "Q08",
                              "text": "50g shock watch was not triggered",
                              "value": False
                          },
                          {
                              "code": "Q09",
                              "text": "25g shock watch present",
                              "value": False
                          },
                          {
                              "code": "Q10",
                              "text": "25g shock watch was not triggered",
                              "value": False
                          },
                          {
                              "code": "Q11",
                              "text": "Desiccant present",
                              "value": False
                          },
                          {
                              "code": "Q12",
                              "text": "Humidity indicator present",
                              "value": False
                          },
                          {
                              "code": "Q13",
                              "text": "Humidity indicator triggered up to percentage",
                              "value": ""
                          },
                          {
                              "code": "Q14",
                              "text": "All items present",
                              "value": False
                          },
                          {
                              "code": "Q15",
                              "text": "Comments",
                              "value": ""
                          }
                      ]
                      }
    }
    return out


def main():
    """Main entry."""
    HELP_LINK="https://itkdb-gtk.docs.cern.ch/receiveShipments.html"

    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg
    IS = ReceiveShipments(client, help_link=HELP_LINK)
    IS.set_accept_focus(True)
    IS.present()
    IS.connect("destroy", Gtk.main_quit)

    try:
        Gtk.main()

    except KeyboardInterrupt:
        print("Arrrgggg!!!")

    dlg.die()


if __name__ == "__main__":
    main()
